/*
*
* XFPROT Copyright (C) 2002-2007 by Tito Ragusa <tito-wolit@tiscali.it>
*
* XFPROT is a graphical frontend for the F-Prot Antivirus
* for Linux Small Business Edition. It was based upon the
* XSTEP-3.5.1 library until version 0.19beta.
* Starting from version 0.20beta it is based upon Gtk-1.2.10.
* From version 1.0-rc2 it is also possible to use Gtk-2.x.x libraries.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation; either version 2 of
* the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* Please read the COPYING and README file!!!
*
* Report bugs to <tito-wolit@tiscali.it>
*
*/

#include <gtk/gtk.h>
#include <fcntl.h>
#include <pwd.h>
#include <signal.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#include <assert.h>
#include <string.h>
#include <fcntl.h>
#include "gunixsignal.h"
#ifdef NO_CORE_DUMPS
#include <sys/prctl.h>
#endif

/* for checkboxes */
static int	flag_nosub = 0;
static int	flag_disinf = 0;
static int	flag_rename = 0;
static int	flag_delete = 0;
static int	flag_append = 0;	
static int	flag_list = 0;
static int	flag_archives = 0;
static int	flag_passwd_archives = 0;
static int	flag_comp_exe = 0;
static int	flag_collect = 0;
static int	flag_ai = 0;
static int	flag_follow = 0;
static int	flag_old = 0;

/* for radiobutton groups */
static int	flag_scanmode = 0;
static int	flag_file_types = 0;
static int	flag_heur = 0;
static int	flag_macro = 0;
static int	flag_macro_action = 0;

static int have_version = 0;
static int saved_settings = 0;
static int aterm_terminal_hack = 0;

static GtkWidget	*main_window;
static GtkWidget	*path_entry_box;
static GtkWidget	*report_path_entry_box;
static GtkWidget	*action_table;
static GtkWidget	*disinfect_button;
static GtkWidget	*rename_button;
static GtkWidget	*delete_button;
static GtkWidget	*macro_remove_table;
static GtkWidget	*scan_1_button;
static GtkWidget	*scan_2_button;
static GtkWidget	*scan_3_button;
static GtkWidget	*macro_1_button;
static GtkWidget	*macro_2_button;
static GtkWidget	*macro_3_button;
static GtkWidget	*heur_1_button;
static GtkWidget	*heur_2_button;
static GtkWidget	*heur_3_button;
static GtkWidget	*ai_check_button;
static GtkWidget	*archives_button;
static GtkWidget	*passwd_archives_button;
static GtkWidget	*radio_button1;
static GtkWidget	*radio_button2;
static GtkWidget	*radio_button3;
static GtkWidget	*radio_button4;
static GtkWidget	*radio_button5;
static GtkWidget	*radio_button6;
static GtkWidget	*check_button1;
static GtkWidget	*check_button2;
static GtkWidget	*check_button3;
static GtkWidget	*check_button4 = NULL;
static GtkWidget	*check_button5;
static GtkWidget	*check_button6;
static GtkWidget	*check_button7;

static char *terminal_emulator = NULL;


/*#################################### File Selection Widget wrappers  ##############################################*/
#ifdef USE_OLD_FILE_SEL
static void do_file_selection(void)
{
	char *path = NULL;

	path = default_old_file_selector(STR_SELECT_PATH_TO_SCAN);

	if (path != NULL) {
		gtk_entry_set_text(GTK_ENTRY(path_entry_box), path);
		xfree(&path);
	}
	/*else  path_entry_box unchanged */
}

#else
/* Circumvent idiotic GtkFileChooser limitation of not being able to open files OR dirs */
static void do_file_selection_file(void)
{
	char *path = NULL;

	path = default_file_selector(STR_SELECT_PATH_TO_SCAN, GTK_FILE_CHOOSER_ACTION_OPEN);

	if (path != NULL) {
		gtk_entry_set_text(GTK_ENTRY(path_entry_box), path);
		xfree(&path);
	}
	/*else  path_entry_box unchanged */
}
static void do_file_selection_dir(void)
{
	char *path = NULL;

	path = default_file_selector(STR_SELECT_PATH_TO_SCAN, GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);

	if (path != NULL) {
		gtk_entry_set_text(GTK_ENTRY(path_entry_box), path);
		xfree(&path);
	}
	/*else  path_entry_box unchanged */
}
#endif

static void do_report_file_selection(void)
{
	char *report_file = NULL;
#ifdef USE_OLD_FILE_SEL
	report_file = default_old_file_selector(STR_SELECT_REPORT_FILE_PATH);
#else
	report_file = default_file_selector(STR_SELECT_REPORT_FILE_PATH, GTK_FILE_CHOOSER_ACTION_OPEN);
#endif
	if (report_file != NULL) {
		gtk_entry_set_text(GTK_ENTRY(report_path_entry_box), report_file);
		xfree(&report_file);
	}
	/*else  report_path_entry_box unchanged */
}

/*############################### End of File Selection Widget wrappers #############################################*/


/*############################### Fprot-info, Virus-list, Update, Test & Reprot button functions ###################*/

static void fprot_info(void)
{
	FILE	*fd;
	int fd_int;
	char *argv_p[3];

	DBG_FUNCTION_NAME;	
	
	argv_p[0] = PATH_TO_F_PROT_BIN;
	argv_p[1] = INFO_CMD2;
	argv_p[2] = NULL;
	
	if (have_version >= 313) {
		/* f-prot version 3.13 and above */
		argv_p[1]=INFO_CMD;
	}
	g_spawn_async_with_pipes (	NULL		/*working_directory*/,
								argv_p		/*argvp */,
								NULL		/*envp*/,
								0			/* flags*/,
								NULL		/* child_setup */,
								NULL		/* user_data */,
								NULL		/* child_pid */,
								NULL		/* standard_input*/,
								&fd_int 	/* standard_output */,
								NULL		/* standard_error */,
								NULL		/* error*/);	
	fd = fdopen(fd_int, "r");
	textview_window_create(STR_INFO, 530, 430, 8, NULL, fd);
	xfclose_nostdin(fd);
}

static void fprot_virlist(void)
{
	FILE	*fd;
	int fd_int;
	char *argv_p[3];
	
	DBG_FUNCTION_NAME;
	
	argv_p[0] = PATH_TO_F_PROT_BIN;
	argv_p[1] = VIRLIST_CMD;
	argv_p[2] = NULL;
	
	g_spawn_async_with_pipes (	NULL		/*working_directory*/,
								argv_p		/*argvp */,
								NULL		/*envp*/,
								0			/* flags*/,
								NULL		/* child_setup */,
								NULL		/* user_data */,
								NULL		/* child_pid */,
								NULL		/* standard_input*/,
								&fd_int 	/* standard_output */,
								NULL		/* standard_error */,
								NULL		/* error*/);	
	/* don't test for NULL here this is done in textview_window_create */
	fd = fdopen(fd_int, "r");
	textview_window_create(STR_VIRUS_LIST, 530, 430, 8, NULL, fd);
	xfclose_nostdin(fd);
}

static void fprot_update(void)
{
	char *cmd;
	DBG_FUNCTION_NAME;

	cmd = xmake_message("%s %s %s", terminal_emulator, SU_SUDO, (have_version >= 410) ? UPDATE_CMD_VER_4_1 : UPDATE_CMD);
	g_spawn_command_line_async (cmd, NULL);
	xfree(&cmd);
}

static void fprot_test(void)
{
	FILE	*fd;
	int fd_int;
	char *argv_p[3];

	DBG_FUNCTION_NAME;
	
	argv_p[0] = PATH_TO_F_PROT_BIN;
	argv_p[1] = VIRTEST_CMD;
	argv_p[2] = NULL;
	
	g_spawn_async_with_pipes (	NULL		/*working_directory*/,
								argv_p		/*argvp */,
								NULL		/*envp*/,
								0			/* flags*/,
								NULL		/* child_setup */,
								NULL		/* user_data */,
								NULL		/* child_pid */,
								NULL		/* standard_input*/,
								&fd_int 	/* standard_output */,
								NULL		/* standard_error */,
								NULL		/* error*/);	
	/* dont test for NULL here this is done in textview_window_create */
	fd = fdopen(fd_int, "r");
	textview_window_create(STR_TEST, 530, 430, 8, NULL, fd);
	xfclose_nostdin(fd);
}

static void view_report(void)
{
	char *report = NULL;

	DBG_FUNCTION_NAME;

	report = g_filename_from_utf8(gtk_entry_get_text(GTK_ENTRY(report_path_entry_box)), -1,NULL,NULL,NULL);
	textedit_window_create(report,530,450,8,report);
	xfree(&report);
}

/*######################### END of Fprot-info, Virus-list, Update, Test & Report button functions ###################*/

/*############################# START of Scan button function  ######################################################*/

static void add_to_command(char **command, char *option)
{
	int size;
	size = xstrlen(option) + xstrlen(*command) + 1;
	*command = (char*)xrealloc(*command, size);
	xstrncat(*command, option,size);
	debug_err_msg("%s(): %s",__FUNCTION__, option);
}

/*#define FORMAT_STRING		"%s %s \"%s\""*/

static void do_scan_f(void)
{
	/*int size;*/
	char *scan_path = NULL;
	char *report_path = NULL;
	char *command = NULL;

	DBG_FUNCTION_NAME;


	scan_path = g_locale_from_utf8(gtk_entry_get_text(GTK_ENTRY(path_entry_box)), -1, NULL, NULL, NULL);

	/* We always "" quote the path to scan to avoid troubles with filenames with spaces
		which are split by the shell and mess up our command line args */

	command = xmake_message("%s %s \"%s\"", terminal_emulator, PATH_TO_F_PROT_BIN, scan_path);
	/* I prefer to create always a report file so no flag checking here.*/
	/* BTW we always quote filenames and paths */
	add_to_command(&command, " -report=");
	add_to_command(&command, "\"");

	report_path = g_locale_from_utf8(gtk_entry_get_text(GTK_ENTRY(report_path_entry_box)), -1, NULL,NULL,NULL);

	add_to_command(&command, report_path);
	add_to_command(&command, "\"");

	/* Let's parse the options*/
	if (flag_scanmode > 0 ) { /* not in report only mode */
		if (IS_SET(flag_disinf))
			add_to_command(&command," -disinf");
		if (IS_SET(flag_rename))
			add_to_command(&command," -rename");
		if (IS_SET(flag_delete))
			add_to_command(&command," -delete");
		
		/* flag_scanmode == 1 is the default: Interactive */
		if (flag_scanmode == 2) /* Automatic */
			add_to_command(&command," -auto");
	}

	switch (flag_file_types) {
		case 0:
			/* "Scan all files" this is the default */
			add_to_command(&command," -dumb");
			break;
		case 1:
			/* "Standard file extensions" */
			add_to_command(&command," -ext");
			break;
		case 2:
			/* "Attempt to identify files" */
			add_to_command(&command," -type");
			break;
		#ifdef DEBUG
		default:
			err_msg("Bad value: flag_file_types");
			exit(EXIT_FAILURE);
		#endif
	}

	if (IS_SET(flag_archives)) {
		/*	f-prot 4.5.3 adds a new option: 
			-archive=n [default is 5].
			Scan inside supported archives n levels deep,
			the supported range is between 1 and 99,
			the default level is 5.
			For the moment we stick with the default.
			TODO
			maybe I'll add a spinbutton in the future.
		*/
		add_to_command(&command," -archive");
		if (have_version >= 440) { 
			/* f-prot version 4.4.1 and above */
			/* Attempts to identify infections within password protected archives. The option -server implies -archive.*/
			if (IS_SET(flag_passwd_archives))
				add_to_command(&command," -server");
			else
				add_to_command(&command," -noserver");	
		}
	} else {
		if (have_version >= 440) { 
			/* f-prot version 4.4.1 and above */
			/*Does not scan inside archives. The option -noarchive implies -noserver.*/
			add_to_command(&command," -noarchive");
			add_to_command(&command," -noserver");	
		}
	}
	 /* Unpack compressed executables. There is no corresponding -nopacked option. 
	 This option is provided for legacy reasons. */
	if (IS_SET(flag_comp_exe))
		add_to_command(&command," -packed");
	if (IS_SET(flag_nosub))
		add_to_command(&command," -nosub");
	if (IS_SET(flag_collect))
		add_to_command(&command," -collect");
	if (IS_SET(flag_list))
		add_to_command(&command," -list");
	if (IS_SET(flag_ai))
		add_to_command(&command," -ai");
	if (IS_SET(flag_old))
		add_to_command(&command," -old");
	/* "Follow symbolic links" is not displayed in f-prot switches but works correctly	*/
	if (IS_SET(flag_follow))
		add_to_command(&command," -follow");
	if (IS_SET(flag_append))
		add_to_command(&command," -append");

	switch (flag_heur) {
		case 1:
			add_to_command(&command," -noheur");
			break;
		case 2:
			add_to_command(&command," -onlyheur");
			break;
	}
	
	switch (flag_macro) {
		/* case 0: scan also for macrovirus is the default */
		case 1:
			add_to_command(&command," -nomacro");
			break;
		case 2:
			add_to_command(&command," -onlymacro");
			break;
	}

	/* not in report only mode */
	if (flag_scanmode > 0 && flag_macro != 1 ) {
		switch (flag_macro_action) {
			case 0:
				/*"Remove infected macros from docs"*/
				add_to_command(&command," -saferemove");
				break;
			case 1:
				/*"Remove all macros from infected docs"*/
				add_to_command(&command," -removenew");
				break;
			case 2:
				/*"Remove all macros from all docs"*/
				add_to_command(&command," -removeall");
				break;
		}
			
	}
	if (/*USE_ATERM_HACK*/ aterm_terminal_hack) {
		add_to_command(&command," -page");
	}
	/*  -wrap switch 	is not supported, if you want to edit the logs use the text editor */
	/*  -nobreak    	is not supported (Do not abort scan if ESC is pressed) */
	/*  -page       	is not supported (Pause after each page.)*/
	/*  -silent     	is not supported (Do not generate any screen output.)*/

	msg("executing scan on: %s", scan_path);
	g_spawn_command_line_async (command, NULL);
	xfree(&scan_path);
	xfree(&report_path);
	xfree(&command);
}

static void scan_f(void)
{
	char *report_path = NULL;
	struct	stat stat_p;

	DBG_FUNCTION_NAME;

	/* Check if at least a flag for actions is set, if not make sure that scanmode is set to report only  and return*/
	if ((IS_NOT_SET(flag_disinf) && IS_NOT_SET(flag_rename) && IS_NOT_SET(flag_delete)) && flag_scanmode > 0 ) {
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (scan_1_button),TRUE);
		SET2(flag_scanmode,0);
		dialog_window(STR_ERROR, ONE_BUTTON, NULL, 1, STR_NO_ACTION_DOING_REPORT_ONLY);
		return;
	}
	
	report_path = g_locale_from_utf8(gtk_entry_get_text(GTK_ENTRY(report_path_entry_box)), -1, NULL,NULL,NULL);

	if (lstat(report_path,&stat_p) == 0) {/* File exists */
		if (!S_ISREG(stat_p.st_mode)) { /* is not a regular file */
			dialog_window(STR_ERROR, ONE_BUTTON, NULL, 1, STR_REPORT_FILE_IS_NOT_A_REGULAR_FILE);
			return;
		} else { /* is a regular file */
			if (IS_NOT_SET(flag_append)) { /* Don't append */
				if (dialog_window(STR_WARNING, TWO_BUTTONS, NULL, 2, STR_REPORT_FILE_EXISTS_OVERWRITE) == 1) {
					/* Don't overwrite */
					return;
				}
				/*else overwrite existing report */
			}
			/*else append to existing report */
		}
	}
	xfree(&report_path);
	/*else file don't exist so write it now */
	do_scan_f();
}

/*############################## End of Scan button function  ######################################################*/

void remove_lock(void)
{
	DBG_FUNCTION_NAME;

	if (remove(LOCK_FILE) == -1) {
		write(STDERR_FILENO, "failed to remove lock file\n", sizeof( "failed to remove lock file\n"));
		dialog_window(STR_ERROR, ONE_BUTTON, NULL, 1, STR_FAILED_TO_REMOVE_LOCK_FILE_ABORTING);
		exit(EXIT_FAILURE);
	}
	write(STDOUT_FILENO, "removed lock file\n", sizeof("removed lock file\n"));
}

static void about_f(void)
{
	FILE	*fd;

	fd = open_file(XFPROT_BIN_DIR LICENSE_FILE);
	about_window_create(VERSION, 620, 430, 8, VERSION"\n"COPYRIGHT"\n"STR_LICENSE"\n"HOMEPAGE"\n"STR_WARRANTY,
						fd,NOT_FIRST_TIME);
	xfclose_nostdin(fd);
}

static void create_file(char * filename)
{
	int pfd;

	if ((pfd = open(filename ,O_RDWR|O_CREAT, 0600)) != -1) {
		close(pfd);
	}
}

static void root_warn(void)
{
	int ret;
	struct stat stat_p;

	DBG_FUNCTION_NAME;

	msg("you are root!");

	if (lstat(NO_ROOT_WARN_FILE,&stat_p) != 0) {
		ret = dialog_window(VERSION" - "STR_WARNING, TWO_BUTTONS_ONE_CHECK, STR_DONT_SHOW_IT_AGAIN,
															2, STR_YOU_ARE_RUNNING_XFPROT_AS_ROOT_CONTINUE);
		switch (ret) {
			case 4: /* No + checkbutton */
				create_file(NO_ROOT_WARN_FILE);
				/* FALLTRHOUGH */
			case 1: /* No */
				remove_lock();
				msg("...exiting");
				exit(EXIT_SUCCESS);
			case 0: /* Ok */
				break;
			case 3: /* Ok + checkbutton */
				create_file(NO_ROOT_WARN_FILE);
		}
	}
}

static void do_lock_file(void)
{
	int pfd;
	int dialog_retval = -1;
	struct	stat stat_p;

	DBG_FUNCTION_NAME;
	
	/* Check if LOCK FILE exists and remove if stale */
	
	/* File exists */
	if (lstat(LOCK_FILE,&stat_p) == 0) {
		err_msg("lock file found");

		dialog_retval = dialog_window(VERSION" - "STR_WARNING, TWO_BUTTONS, NULL, 2, STR_XFPROT_HAS_DETECTED_A_LOCK_FILE);
		if (dialog_retval == 1) {
			msg("...exiting");
			exit(EXIT_SUCCESS);
		}
		remove_lock();
	}
	/* cannot open */
	if ((pfd = open(LOCK_FILE,O_RDWR|O_CREAT|O_EXCL,0640)) < 0) {
		err_msg("cannot open lock file: %s", strerror(errno));
		dialog_window(VERSION" - "STR_ERROR, ONE_BUTTON, NULL, 1, STR_CANNOT_OPEN_LOCK_FILE_ABORTING);
		exit(EXIT_FAILURE);
	}
	/* cannot lock */
	if (lockf(pfd,F_TLOCK,0) < 0) {
		err_msg("lockf(): %s", strerror(errno));
		dialog_window(VERSION" - "STR_ERROR, ONE_BUTTON, NULL, 1, STR_CANNOT_LOCK_FILE_ABORTING);
		exit(EXIT_FAILURE);
	}
	/* only first instance continues */
	close(pfd);
}

static void set_dir_properties_and_cd_in(uid_t uid, gid_t gid)
{
	DBG_FUNCTION_NAME;

	msg("setting permissions of private dir to 0700");
	if (xchmod(XFPROT_PRIV_DIR,0700, RETURN_ON_FAILURE) == -1) {
		dialog_window(VERSION" - "STR_ERROR, ONE_BUTTON, NULL, 1, STR_CANNOT_SET_PERMISSIONS_OF_PRIVATE_DIR_ABORTING);
		exit(EXIT_FAILURE);
	}

	msg("setting owner of private dir");
	if (xchown(XFPROT_PRIV_DIR, uid, gid, RETURN_ON_FAILURE) == -1) {
		dialog_window(VERSION" - "STR_ERROR, ONE_BUTTON, NULL, 1, STR_CANNOT_SET_OWNER_OF_PRIVATE_DIR_ABORTING);
		exit(EXIT_FAILURE);
	}

	msg("chdir to private dir");
	if (xchdir(XFPROT_PRIV_DIR, RETURN_ON_FAILURE) == -1) {
		dialog_window(VERSION" - "STR_ERROR, ONE_BUTTON, NULL, 1, STR_CANNOT_CHANGE_DIR_TO_PRIVATE_DIR_ABORTING);
		exit(EXIT_FAILURE);
	}
}

void check_for_bin(void)
{
	DBG_FUNCTION_NAME;

	/* access  may  not  work  correctly  on NFS file systems */
	if (access(PATH_TO_F_PROT_BIN , X_OK) < 0) {
		err_msg("access(): %s: %s",PATH_TO_F_PROT_BIN, strerror(errno));
		dialog_window(VERSION" - "STR_ERROR, ONE_BUTTON, NULL, 1, PATH_TO_F_PROT_BIN"\n"STR_FILE_NOT_FOUND_ABORTING);
		exit(EXIT_FAILURE);
	}
}

void toggle1_f(void)
{
	DBG_FUNCTION_NAME;
	printf("flag_scanmode %d flag_macro %d\n", flag_scanmode, flag_macro);
	if (flag_scanmode > 0 && flag_macro < 2 ) {
		/* It's better to reset the flags to default values
			here to avoid leftovers from previous toggles */
		gtk_widget_show(action_table);
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (disinfect_button),	TRUE);
		SET(flag_disinf);
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rename_button),	FALSE);
		RESET(flag_rename);
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (delete_button),	FALSE);
		RESET(flag_delete);
	} else {
		gtk_widget_hide(action_table);
		debug(err_msg("hiding widget action_table"));
	}
}

static void toggle2_f(void)
{
	DBG_FUNCTION_NAME;
	printf("flag_scanmode %d flag_macro %d\n", flag_scanmode, flag_macro);
	if ( flag_scanmode > 0 && flag_macro != 1 ) {
		/* It's better to reset the flags to default values
			here to avoid leftovers from previous toggles */
		gtk_widget_show(macro_remove_table);
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (macro_1_button), TRUE);
		SET2(flag_macro_action,0);
	} else {
		gtk_widget_hide(macro_remove_table);
		debug(err_msg("hiding widget macro_remove_table"));
	}
}

static void flag_scanmode_callback(GtkWidget *widget, gpointer callback_data)
{
	assert(widget != NULL);

	SET2(flag_scanmode,atoi(callback_data));
	toggle1_f();
	toggle2_f();
}

static void flag_file_types_callback(GtkWidget *widget, gpointer callback_data)
{
	assert(widget != NULL);

	SET2(flag_file_types,atoi(callback_data));
}

static void flag_macro_callback(GtkWidget *widget, gpointer callback_data)
{
	assert(widget != NULL);

	SET2(flag_macro,atoi(callback_data));
	toggle2_f();
	toggle1_f();
}

static void flag_macro_action_callback(GtkWidget *widget, gpointer callback_data)
{
	assert(widget != NULL);

	SET2(flag_macro_action,atoi(callback_data));
}

static void flag_heur_callback(GtkWidget *widget, gpointer callback_data)
{
	assert(widget != NULL);

	SET2(flag_heur, atoi(callback_data));
	/* Disable neural-network virus detection. The -ai option should not be used with the -noheur option.*/
	if (flag_heur == 1) {
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (ai_check_button), FALSE);
	}
}

static void flag_ai_callback(GtkWidget *widget, int *callback_data)
{
	DBG_FUNCTION_NAME;
	assert(widget != NULL);

	if (IS_SET(*callback_data)) {
		RESET(*callback_data);
	} else {
		/* Enable ai. The -ai option (neural-network virus detection) should not be used with the -noheur option.*/
		if (flag_heur == 1) {
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (heur_1_button), TRUE);
			SET2(flag_heur, 0);
		}
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (ai_check_button),TRUE);
		SET(*callback_data);	
	}
}

static void flag_archives_callback(GtkWidget *widget, int *callback_data)
{
	assert(widget != NULL);

	if (IS_SET(*callback_data)) { 
		RESET(*callback_data);
		if (have_version >= 440 ) { /* f-prot version 4.4.1 and above */
			/* check for flag_protected_archives and disable if set */
			if (IS_SET(flag_passwd_archives)) {
				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (passwd_archives_button),FALSE);
				SET2(flag_passwd_archives,0);
			}
		}
	} else {
		SET(*callback_data);	
	}
}

static void flag_passwd_archives_callback(GtkWidget *widget, int *callback_data)
{
	assert(widget != NULL);
	/* f-prot version 4.4.1 and above */
	if (IS_SET(*callback_data)) {
		RESET(*callback_data);
	} else {
		/* check for flag_archives and enable if unset */
		if (IS_NOT_SET(flag_archives)) {
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (archives_button),TRUE);
			SET2(flag_archives, 1);
		}
		SET(*callback_data);	
	}
}

static void flag_callback(GtkWidget *widget, int *callback_data)
{
	assert(widget != NULL);

	if (IS_SET(*callback_data))
		RESET(*callback_data);
	else
		SET(*callback_data);	
}

/* This callback quits the program */
static void main_window_close(void)
{
	DBG_FUNCTION_NAME;

	gtk_main_quit ();
	gtk_widget_destroy(main_window);
	remove_lock();
	write(STDOUT_FILENO, "...exiting", sizeof("...exiting"));	
}

void terminate(gint sig, gpointer data) __attribute__(( __noreturn__ ));
void terminate(gint sig, gpointer data)
{
	switch (sig) {
		case SIGINT:
			write(STDOUT_FILENO, "SIGINT signal catched\n", sizeof("SIGINT signal catched\n"));	
			break;
		case SIGTERM:
			write(STDOUT_FILENO, "SIGTERM signal catched\n", sizeof("SIGTERM signal catched\n"));	
			break;
		case SIGHUP:
			write(STDOUT_FILENO, "SIGHUP signal catched\n", sizeof("SIGHUP signal catched\n"));	
			break;
	}
	main_window_close();
	exit(EXIT_FAILURE);
	/* to shut a compiler warning */
	data = NULL;
}

static void save_settings(void)
{
	FILE *fd;
	mode_t  saved_mask = umask(077);

	if ((fd = fopen(XFPROT_CONFIG_FILE, "w")) != NULL) {
			fprintf(fd, "# %s - %s\n# Configuration file\n#\n\n", VERSION, COPYRIGHT); 
			fprintf(fd, "scanmode=%d\n",     flag_scanmode);
			fprintf(fd, "macro=%d\n",        flag_macro);
			fprintf(fd, "macro_action=%d\n", flag_macro_action);
			fprintf(fd, "file_types=%d\n",   flag_file_types);
			fprintf(fd, "heur=%d\n",         flag_heur);
			fprintf(fd, "nosub=%d\n",        flag_nosub);
			fprintf(fd, "disinf=%d\n",       flag_disinf);
			fprintf(fd, "rename=%d\n",       flag_rename); 
			fprintf(fd, "delete=%d\n",       flag_delete);
			fprintf(fd, "append=%d\n",       flag_append);
			fprintf(fd, "list=%d\n",         flag_list);
			fprintf(fd, "archives=%d\n",     flag_archives);
			if (have_version >= 440) {
				fprintf(fd, "arch_passwd=%d\n", flag_passwd_archives);
			} else {
				/* comp_exe only for f-prot version prior to 4.4.1 */
				fprintf(fd, "comp_exe=%d\n", flag_comp_exe); 
			}
			fprintf(fd, "collect=%d\n",      flag_collect);
			fprintf(fd, "ai=%d\n",           flag_ai);
			fprintf(fd, "follow=%d\n",       flag_follow);
			fprintf(fd, "old=%d\n",          flag_old);

			if (fflush(fd) || fsync(fileno(fd)) || fclose(fd)) {
				dialog_window(VERSION" - "STR_ERROR, ONE_BUTTON, NULL, 1, STR_ERROR_WRITING_FILE_S_S,
									XFPROT_CONFIG_FILE, strerror(errno));
			} else {
				dialog_window(VERSION, ONE_BUTTON, NULL, 1, STR_SETTINGS_SAVED);		
			}
	} else {
		dialog_window(VERSION" - "STR_ERROR, ONE_BUTTON, NULL, 1,STR_CANNOT_OPEN_FILE":\n"XFPROT_CONFIG_FILE);
	}
	umask(saved_mask);
}

static GtkActionEntry xfprot_entries[] = {
	/* name,		stock id,	label,				accelerator,		tooltip,	callback */
	{ "FileMenu",	NULL,		MNU_STR_FILE,			NULL,				NULL,		NULL},
	{ "ToolsMenu",	NULL,		MNU_STR_TOOLS,			NULL,				NULL,		NULL},
	{ "HelpMenu",	NULL,		MNU_STR_HELP,			NULL,				NULL,		NULL},
	{ "Scan",		NULL,		MNU_STR_SCAN, 			"F1",				NULL,		G_CALLBACK (scan_f) },
	{ "Update",		NULL,		MNU_STR_UPDATE,			"F2",				NULL,		G_CALLBACK (fprot_update) },
	{ "Report",		NULL,		MNU_STR_REPORT,			"F3",				NULL,		G_CALLBACK (view_report) },
	{ "Quit",		NULL,		MNU_STR_QUIT,			SHC_FILE_QUIT,		NULL,		G_CALLBACK (main_window_close) },
	{ "Test",		NULL,		MNU_STR_TEST,			"F4",				NULL,		G_CALLBACK (fprot_test) },
	{ "Info",		NULL,		MNU_STR_INFO,			"F5",				NULL,		G_CALLBACK (fprot_info) },
	{ "VirusList",	NULL,		MNU_STR_VIRUS_LIST,		"F6",				NULL,		G_CALLBACK (fprot_virlist) },
	{ "SaveSettings",NULL,		MNU_STR_SAVE_SETTINGS,	"F7",				NULL,		G_CALLBACK (save_settings) },
	{ "About",		NULL,		MNU_STR_ABOUT,			SHC_HELP_ABOUT,		NULL,		G_CALLBACK (about_f) },
};

static const gchar *xfprot_ui_info = 
"<ui>"
"  <menubar name='MenuBar'>"
"    <menu action='FileMenu'>"
"      <menuitem action='Scan'/>"
"      <menuitem action='Update'/>"
"      <menuitem action='Report'/>"
"      <separator/>"
"      <menuitem action='Quit'/>"
"    </menu>"
"    <menu action='ToolsMenu'>"
"      <menuitem action='Test'/>"
"      <menuitem action='Info'/>"
"      <menuitem action='VirusList'/>"
"      <menuitem action='SaveSettings'/>"
"    </menu>"
"    <menu action='HelpMenu'>"
"      <menuitem action='About'/>"
"    </menu>"
"  </menubar>"
"</ui>";

static struct MenuEntry xfprot_menu_entries[] = {
{ xfprot_entries,		NULL,						NULL,			G_N_ELEMENTS (xfprot_entries),				0, NULL},
{ NULL,					NULL,						NULL,			0,											0, NULL},
/* Set last to NULL */ 
};

int read_saved_settings(void)
{
	FILE *fd;
	int ret = 0;
	char *str;

	DBG_FUNCTION_NAME;

	if ((fd = fopen(XFPROT_CONFIG_FILE, "r")) != NULL) {
		msg("reading config from: %s", XFPROT_CONFIG_FILE);
		/* Read the config file */
		while ((str = get_chomped_line_from_file(fd)) != NULL) {
			/* Trim leading and triling whitespace */
			trim(str);
			debug_err_msg("%s", str);
			/* Skip commented or empty lines */
			if (str[0] == '#' || str[0] == '\0') continue;

			/* Set the config values */
			if (strncmp("scanmode=", str, 9) == 0)         flag_scanmode        =    atoi(&str[ 9]);
			if (strncmp("macro=", str, 6) == 0)            flag_macro           =    atoi(&str[ 6]);
			if (strncmp("macro_action=",    str, 13) == 0) flag_macro_action    =    atoi(&str[13]);
			if (strncmp("heur=",            str,  5) == 0) flag_heur            =    atoi(&str[ 5]);
			if (strncmp("file_types=",      str, 11) == 0) flag_file_types      =    atoi(&str[11]);
			if (strncmp("nosub=",           str,  6) == 0) flag_nosub           = (str[ 6] == '1');
			if (strncmp("disinf=",          str,  7) == 0) flag_disinf          = (str[ 7] == '1');
			if (strncmp("rename=",          str,  7) == 0) flag_rename          = (str[ 7] == '1');
			if (strncmp("delete=",          str,  7) == 0) flag_delete          = (str[ 7] == '1');
			if (strncmp("append=",          str,  7) == 0) flag_append          = (str[ 7] == '1');
			if (strncmp("list=",            str,  5) == 0) flag_list            = (str[ 5] == '1');
			if (strncmp("archives=",        str,  9) == 0) flag_archives        = (str[ 9] == '1');
			if (strncmp("ai=",              str,  3) == 0) flag_ai              = (str[ 3] == '1');
			if (have_version >= 440) {
				if (strncmp("arch_passwd=", str, 12) == 0) flag_passwd_archives = (str[12] == '1');
			} else {
				if (strncmp("comp_exe=",    str,  9) == 0) flag_comp_exe        = (str[ 9] == '1');
			}
			if (strncmp("collect=",         str, 8) == 0)  flag_collect         = (str[ 8] == '1');
			if (strncmp("follow=",          str, 7) == 0)  flag_follow          = (str[ 7] == '1');
			if (strncmp("old=",             str, 4) == 0)  flag_old             = (str[ 4] == '1');
		}
		ret++;
	}
	debug_err_msg("restore saved_settings %d", ret);
	return ret;
}

/* List of terminal emulators known to work */
char *terms[] = {
	"Eterm",          "--pause --exec", "0",
	"konsole",        "--noclose -e",   "0",
	"xterm",          "-hold -e",       "0",
	"uxterm",         "-hold -e",       "0", /*Not tested */
	"mrxvt",          "-hold -e",       "0", /*Not tested */
	"aterm",          "-e",             "1",
	"multi-aterm",    "-e",             "1", /*Not tested */
	"rxvt",           "-e",             "1",
	"Terminal",       "-H -x",          "0",
	"gnome-terminal", "-e",             "1", /*Not tested */
	NULL,       NULL,             NULL
};

/* Copyright (C) 2006 Gabriel Somlo <somlo at cmu.edu> */
int execable_file(const char *name)
{
	struct stat s;
	return (!access(name, X_OK) && !stat(name, &s) && S_ISREG(s.st_mode));
}
/* Copyright (C) 2006 Gabriel Somlo <somlo at cmu.edu> */
char *find_execable(char *filename)
{
	char *path, *p, *n;

	p = path = xstrdup(getenv("PATH"));
	while (p) {
		n = strchr(p, ':');
		if (n)
			*n++ = '\0';
		if (*p != '\0') { /* it's not a PATH="foo::bar" situation */
			p = xconcat_path_file(p, filename);
			if (execable_file(p)) {
				free(path);
				return p;
			}
			free(p);
		}
		p = n;
	}
	free(path);
	return NULL;
}

void autodetect_terminal_emulator(void)
{
	int i;
	char *path = NULL;

	terminal_emulator = NULL;

	if (!getenv("PATH")) setenv("PATH", "/bin:/sbin:/usr/bin:/usr/sbin:/usr/local/bin", 1);
	
	for (i = 0; terms[i] != NULL; i += 3) {
		debug_err_msg("%s", terms[i]);
		if ((path = find_execable(terms[i]))) {
			terminal_emulator = xmake_message("%s %s", path, terms[i + 1]);
			xfree(&path);
			if (terms[i + 2][0] == '1') aterm_terminal_hack = 1;
			break;
		}
	}

	if (!terminal_emulator) {
		dialog_window(VERSION" - "STR_ERROR, ONE_BUTTON, NULL, 1, "Terminal emulator: not found");
		remove_lock();
		exit(EXIT_FAILURE);
	}
	debug_err_msg("%s", terminal_emulator);
}

int main(int argc, char **argv)
{
	GtkWidget		*label;
	GtkWidget		*top_table;
	GtkWidget		*mode_table;
	GtkWidget		*macro_table;
	GtkWidget		*file_types_table;
	GtkWidget		*report_table;
	GtkWidget		*option_table;
	GtkWidget		*frame1;
	GtkWidget		*frame2;
	GtkWidget		*frame3;
	GtkWidget		*frame4;
	GtkWidget		*frame5;
	GtkWidget		*frame6;
	GtkWidget		*frame7;
	GtkWidget		*table1;
	GtkWidget		*button1;
	GtkWidget		*button2;
	GtkWidget		*button3;
	GtkWidget		*button4;
	GtkWidget		*button5;
	GtkWidget		*button6;
	GtkWidget		*button7;
	GtkWidget		*button8;
#ifndef USE_OLD_FILE_SEL
	GtkWidget		*button9;
#endif
	GtkWidget		*menubar;
#ifndef USE_NEW_TOOLTIP
	GtkTooltips		*action_tooltip;
	GtkTooltips		*onlyheur_tooltip;
#endif
	GSList			*scan_mode_group = NULL;
	GSList			*scan_macro_group = NULL;
	GSList			*file_types_group = NULL;
	GSList			*remove_macro_group = NULL;
	GSList			*heuristics_group = NULL;
	char *command;
	char tmp[64];
	char *path = NULL;
	char *report_file_path = NULL;
	FILE	*fd;

	struct	passwd *pw;
	struct	stat stat_p;

	/* print disclaimer to terminal */
	xputs("\n"VERSION"\n\n"COPYRIGHT"\n"STR_LICENSE"\n"HOMEPAGE"\n"STR_WARRANTY"\n\n");
	msg("starting...");

	unixsignal_init();
	unixsignal_install(SIGHUP, terminate, NULL);
	unixsignal_install(SIGTERM, terminate, NULL);
	unixsignal_install(SIGINT, terminate, NULL);

	gtk_init (&argc, &argv);

	/* Force the output to be unbuffered when debugging.*/
	debug(setbuf(stdout, (char *) 0));

	/* Command line args. No dialog windows on errors as this is console stuff */
	if (argc == 2) {
		if (strcmp(argv[1],"-v") == 0) {
			msg(VERSION);
			exit(EXIT_SUCCESS);
		}
		err_msg("unknown command line switch: %s",argv[1]);
		exit(EXIT_FAILURE);
	}

	autodetect_terminal_emulator();

	if (argc > 2) {
		if (strcmp(argv[1], "--kde-menu-action") == 0) {
			command = xmake_message("%s %s \"%s\"", terminal_emulator, PATH_TO_F_PROT_BIN, argv[2]);
			msg("executing scan on: %s", argv[2]);
			g_spawn_command_line_async (command, NULL);
			xfree(&command);
			exit(EXIT_SUCCESS);
		} else {
			err_msg("too much args on command line");
			exit(EXIT_FAILURE);
		}
	}

	#ifdef NO_CORE_DUMPS
	/* make the process not dumpable under Linux 2.4, seems to work also for 2.6.0-test3 */
	prctl(PR_SET_DUMPABLE,0);
	msg("process is%sdumpable",(prctl(PR_GET_DUMPABLE)==0)?" not ":" ");
	#endif

	/* Try to determine the home dir of the user in
		passwd struct to avoid use of getenv("HOME").
		Create a private xfprot dir in user's home dir
		if it doesn't already exist.
		Set explicitly ownership and permissions of xfprot's dir.
		If something goes wrong we issue an error and exit.
		BTW: the user is forced to write the report always in
		xfprot's  private dir.
	*/

	/* Who am I ? */
	if ((pw = getpwuid(getuid())) == NULL) {
		debug_err_msg("getpwuid(): %s", strerror(errno));
		dialog_window(VERSION" - "STR_ERROR, ONE_BUTTON, NULL, 1, STR_CANNOT_GET_PWENT_ABORTING);
		exit(EXIT_FAILURE);
	}
	/* chdir to user's home dir */
	if ((xchdir(pw->pw_dir, RETURN_ON_FAILURE)) == EXIT_ERROR) {
		dialog_window(VERSION" - "STR_ALERT, ONE_BUTTON, NULL, 1, STR_FAILED_TO_CD_TO_HOMEDIR_TRYING_ROOTDIR);
		if (xchdir("/",RETURN_ON_FAILURE) == -1) {
			dialog_window(VERSION" - "STR_ERROR, ONE_BUTTON, NULL, 1, STR_FAILED_TO_CD_TO_ROOTDIR_ABORTING);
			exit(EXIT_FAILURE);
		}
	}
	/* Check for xfprot's private dir */
	if (lstat(XFPROT_PRIV_DIR,&stat_p) == 0) {
		msg("private dir found");
		if (S_ISLNK(stat_p.st_mode)) {
			/* We don't want to be redirected by a link */
			debug(err_msg("private dir is a link"));
			dialog_window(VERSION" - "STR_ERROR, ONE_BUTTON, NULL, 1, STR_PRIVATE_DIR_IS_A_LINK_ABORTING);
			exit(EXIT_FAILURE);
		}
	} else {
		err_msg("private dir not found");
		if (xmkdir(XFPROT_PRIV_DIR, 0700, RETURN_ON_FAILURE) == -1) {
			dialog_window(VERSION" - "STR_ERROR, ONE_BUTTON, NULL, 1, STR_CANNOT_CREATE_PRIVATE_DIR_ABORTING);
			exit(EXIT_FAILURE);
		}
	}

	/* Explicitly set private dir's permissions and owner and chdir into it */
	set_dir_properties_and_cd_in(pw->pw_uid, pw->pw_gid);
	/* check for f-prot*/
	check_for_bin();
	/* lock file stuff */
	do_lock_file();
	/* display a warning if we are root */
	if (pw->pw_uid == 0) root_warn();

	/* Detect f-prot version 4.4.x for -server/-noserver switch and detect -verno switch */
	fd = xpopen(PATH_TO_F_PROT_BIN" "INFO_CMD,"r");
	while (fgets(tmp, sizeof(tmp), fd) != NULL) {
		/* Detect -verno switch f-prot >=3.13 */
		if (strstr(tmp,"F-PROT ANTIVIRUS") != NULL)
			SET2(have_version, 313); /* f-prot version 3.13 */
		/* Detect version */
		if (strstr(tmp,"Program version: 4.1") != NULL)
			SET2(have_version, 410); /* f-prot version 4.1.x */
		if (strstr(tmp,"Program version: 4.4") != NULL)
			SET2(have_version, 440); /* f-prot version 4.4.x */
		if (strstr(tmp,"Program version: 4.5") != NULL)
			SET2(have_version, 450); /* f-prot version 4.5.x */
		if (strstr(tmp,"Program version: 4.6") != NULL)
			SET2(have_version, 460); /* f-prot version 4.6.x */
	}
	xpclose_nostdin(fd);

	debug_err_msg(" f-prot version %d", have_version);

	if (have_version == 0 ) {
		fd = xpopen(PATH_TO_F_PROT_BIN" "INFO_CMD,"r");
		textview_window_create(STR_ERROR, 530, 430, 8, NULL, fd);
		xpclose_nostdin(fd);
		remove_lock();
		unixsignal_cleanup();
		exit(EXIT_FAILURE);
	}

	saved_settings = read_saved_settings();

	main_window = window_create(640,  450, 0, VERSION,GTK_SIGNAL_FUNC(main_window_close), NULL, MODAL);

	table1 = table_create(10, 21, main_window, 2 ,  NOT_HOMOGENEUS);
	menubar = main_menu_create(GTK_WIDGET (main_window), xfprot_menu_entries, xfprot_ui_info, NULL);
	gtk_table_attach_defaults (GTK_TABLE(table1),menubar, 0, 10, 0, 1);
	
	frame1 = frame_create_on_table(STR_PATH_TO_SCAN, table1, 0, 10, 1,4);
	top_table = table_create(20, 2, frame1, 2 , HOMOGENEUS);
	path = xgetcwd(path);

	debug_err_msg("%s", path);

	path_entry_box = default_text_box_create_on_table(path, EDITABLE, top_table, 0, 19, 0, 1);
#ifdef USE_OLD_FILE_SEL
	button1 = button_create_on_table(":",GTK_SIGNAL_FUNC(do_file_selection), NULL, top_table, 19, 20, 0,1);
#else
	button1 = button_create_on_table(STR_FILE,GTK_SIGNAL_FUNC(do_file_selection_file), NULL, top_table, 19, 20, 0,1);
	button9 = button_create_on_table(STR_DIR, GTK_SIGNAL_FUNC(do_file_selection_dir), NULL, top_table, 20, 21, 0,1);
#endif

	check_button1 = check_create_on_table(STR_NOSUB, GTK_SIGNAL_FUNC(flag_callback), &flag_nosub, 
										INACTIVE, top_table, 0, 10, 1, 2);

	frame2 = frame_create_on_table(STR_SCAN_MODE, table1, 0, 5, 4, 8);
	mode_table = table_create(1, 3, frame2, 2 , HOMOGENEUS);

	/* Warning we can't use the ACTIVE flag here 'cause the callback function acts on widgets
		we haven't created yet, so we do things manually later by calling toggle1_f() */
	scan_1_button = radio_create_on_table(NULL, STR_REPORT_ONLY, GTK_SIGNAL_FUNC(flag_scanmode_callback), "0",
													INACTIVE, mode_table, 0, 1, 0, 1);
	scan_mode_group = gtk_radio_button_get_group(GTK_RADIO_BUTTON (scan_1_button));
	scan_2_button = radio_create_on_table(scan_1_button,STR_INTERACTIVE,GTK_SIGNAL_FUNC(flag_scanmode_callback),"1",
													INACTIVE,mode_table, 0, 1, 1,2);
	scan_3_button = radio_create_on_table(scan_2_button, STR_AUTOMATIC,GTK_SIGNAL_FUNC(flag_scanmode_callback),"2",
													INACTIVE,mode_table, 0, 1, 2,3);

	frame3 = frame_create_on_table( STR_SCAN_FOR_MACRO_VIRUS, table1, 5, 10, 4, 8);
	macro_table = table_create(1, 3, frame3, 2 ,  HOMOGENEUS);
	/* Warning we can't use the ACTIVE flag here because the callback function acts on widgets
		we haven't created yet, so we do things manually */
	radio_button1 = radio_create_on_table(NULL, STR_YES, GTK_SIGNAL_FUNC(flag_macro_callback),
										"0", INACTIVE, macro_table, 0, 1, 0, 1);
	scan_macro_group = gtk_radio_button_get_group(GTK_RADIO_BUTTON (radio_button1));
	radio_button2 = radio_create_on_table(radio_button1,STR_NO,GTK_SIGNAL_FUNC(flag_macro_callback),
										"1", INACTIVE, macro_table, 0, 1, 1, 2);
	radio_button3 = radio_create_on_table(radio_button2,STR_ONLY,GTK_SIGNAL_FUNC(flag_macro_callback),
										"2", INACTIVE, macro_table, 0, 1, 2, 3);
	
	frame4 = frame_create_on_table( STR_ACTIONS_TO_TRY_ON_INFECTED_FILES, table1, 0, 5, 8, 12);
	action_table = table_create(1, 3, frame4, 2 ,  HOMOGENEUS);

#ifndef USE_NEW_TOOLTIP
	action_tooltip = gtk_tooltips_new();
#endif

	disinfect_button = check_create_on_table(	STR_DISINFECT_WHENEVER_POSSIBLE,
												GTK_SIGNAL_FUNC(flag_callback), &flag_disinf,
												INACTIVE, action_table, 0, 1, 0, 1);
#ifdef USE_NEW_TOOLTIP
	gtk_widget_set_tooltip_text(disinfect_button, STR_COMBINATIONS_OF_SWITCHES);
#else
	gtk_tooltips_set_tip (GTK_TOOLTIPS (action_tooltip),disinfect_button, STR_COMBINATIONS_OF_SWITCHES,NULL);
#endif

	rename_button =		check_create_on_table(	STR_RENAME_INFECTED_FILES,
												GTK_SIGNAL_FUNC(flag_callback), &flag_rename,
												INACTIVE, action_table, 0, 1, 1, 2);
#ifdef USE_NEW_TOOLTIP
	gtk_widget_set_tooltip_text(rename_button, STR_COMBINATIONS_OF_SWITCHES);
#else
	gtk_tooltips_set_tip (GTK_TOOLTIPS (action_tooltip),rename_button, STR_COMBINATIONS_OF_SWITCHES, NULL);
#endif

	delete_button =		check_create_on_table(	STR_DELETE_INFECTED_FILES,
												GTK_SIGNAL_FUNC(flag_callback), &flag_delete,
												INACTIVE, action_table, 0, 1, 2, 3);
#ifdef USE_NEW_TOOLTIP
	gtk_widget_set_tooltip_text(delete_button, STR_COMBINATIONS_OF_SWITCHES);
#else
	gtk_tooltips_set_tip (GTK_TOOLTIPS (action_tooltip), delete_button, STR_COMBINATIONS_OF_SWITCHES, NULL);
	gtk_tooltips_enable (action_tooltip);
#endif

	frame5 = frame_create_on_table(STR_MACRO_VIRUS_REMOVAL_OPTIONS, table1, 5, 10, 8, 12);
	macro_remove_table = table_create(1, 3, frame5, 2 ,  HOMOGENEUS);

	macro_1_button = radio_create_on_table(	NULL, STR_REMOVE_INFECTED_MACROS_FROM_DOCS,
											GTK_SIGNAL_FUNC(flag_macro_action_callback), "0",
											INACTIVE, macro_remove_table, 0, 1, 0, 1);
	remove_macro_group = gtk_radio_button_get_group(GTK_RADIO_BUTTON (macro_1_button));
	macro_2_button = radio_create_on_table(	macro_1_button,STR_REMOVE_ALL_MACROS_FROM_INFECTED_DOCS,
													GTK_SIGNAL_FUNC(flag_macro_action_callback), "1",
													INACTIVE, macro_remove_table, 0, 1, 1, 2);
	macro_3_button = radio_create_on_table(	macro_2_button,STR_REMOVE_ALL_MACROS_FROM_ALL_DOCS,
													GTK_SIGNAL_FUNC(flag_macro_action_callback), "2",
													INACTIVE, macro_remove_table, 0, 1, 2, 3);

	frame6 = frame_create_on_table(STR_FILE_TYPES, table1, 0, 5, 12, 16);
	file_types_table = table_create(1, 3, frame6, 2 ,  HOMOGENEUS);

	radio_button4 = radio_create_on_table(NULL,	STR_SCAN_ALL_FILES,
										GTK_SIGNAL_FUNC(flag_file_types_callback), "0",
										INACTIVE, file_types_table, 0, 1, 0, 1);
	file_types_group = gtk_radio_button_get_group(GTK_RADIO_BUTTON (radio_button4));
	radio_button5 = radio_create_on_table(radio_button4, STR_STANDARD_FILE_EXTENSIONS,
										GTK_SIGNAL_FUNC(flag_file_types_callback), "1",
										INACTIVE, file_types_table, 0, 1, 1, 2);
	radio_button6 = radio_create_on_table(radio_button5, STR_ATTEMPT_TO_IDENTIFY_FILES,
										GTK_SIGNAL_FUNC(flag_file_types_callback), "2",
										INACTIVE, file_types_table, 0, 1, 2, 3);
										
	frame6 = frame_create_on_table(STR_REPORT_FILE, table1, 0, 5, 16, 20);
	report_table = table_create(8, 3, frame6, 2 , HOMOGENEUS);

	report_file_path=xconcat_path_file(path, DEF_REPORT_FILE);

	report_path_entry_box = default_text_box_create_on_table( report_file_path , EDITABLE, report_table, 0, 7, 0, 1);
	button2 = button_create_on_table(STR_FILE ,GTK_SIGNAL_FUNC(do_report_file_selection), NULL, report_table, 7, 8, 0, 1);

	xfree(&report_file_path);
	xfree(&path);

	check_button2 = check_create_on_table(STR_APPEND_TO_EXISTING_REPORT_FILE,
										GTK_SIGNAL_FUNC(flag_callback), &flag_append,
										INACTIVE, report_table, 0, 7, 1, 2);
	check_button3 = check_create_on_table(STR_LIST_ALL_SCANNED_FILES,
										GTK_SIGNAL_FUNC(flag_callback), &flag_list,
										INACTIVE, report_table, 0, 7, 2, 3);

	frame7 = frame_create_on_table(STR_OPTIONS, table1, 5, 10, 12, 20);
	option_table = table_create(5, 7, frame7, 2 , HOMOGENEUS);

	archives_button = check_create_on_table(STR_SCAN_INSIDE_ARCHIVES,
										GTK_SIGNAL_FUNC(flag_archives_callback), &flag_archives,
										INACTIVE, option_table, 0, 5, 0, 1);
	

	if (have_version >= 440 ) {  /* f-prot version 4.4.1  and above */
		/* -packed is default in f-prot 4.4.1 and above so we don't need a checkbox for it
		 as a nopacked options doesn't exist and we can use this place for this new option */
		/* this option depends on flag_archives */
		passwd_archives_button = check_create_on_table( STR_SCAN_PROTECTED_ARCHIVES,
										GTK_SIGNAL_FUNC(flag_passwd_archives_callback), &flag_passwd_archives,
										INACTIVE, option_table, 0, 5, 1, 2);
	} else {
		check_button4 = check_create_on_table(STR_SCAN_COMPRESSED_EXECUTABLES,
										GTK_SIGNAL_FUNC(flag_callback), &flag_comp_exe,
										INACTIVE, option_table, 0, 5, 1, 2);
	}
	
	check_button5 = check_create_on_table(STR_SCAN_FOR_OTHER_VARIOUS_MALWARE,
										GTK_SIGNAL_FUNC(flag_callback), &flag_collect,
										INACTIVE, option_table, 0, 5, 2, 3);

	/* depends on flag_heur run callback only after all widgets are created */
	ai_check_button = check_create_on_table(STR_ENABLE_NEURAL_NETWORK_VIRUS_DETECTION,
										GTK_SIGNAL_FUNC(flag_ai_callback), &flag_ai,
										INACTIVE, option_table, 0, 5, 3, 4);

	check_button6 = check_create_on_table(STR_FOLLOW_SYMBOLIC_LINKS,
										GTK_SIGNAL_FUNC(flag_callback), &flag_follow,
										INACTIVE, option_table, 0, 5, 4, 5);
	check_button7 = check_create_on_table(STR_DO_NOT_COMPLAIN_ABOUT_OLD_DEF_FILES,
										GTK_SIGNAL_FUNC(flag_callback), &flag_old,
										INACTIVE, option_table, 0, 5, 5, 6);

	/* depends on flag_ai run callback only after all widgets are created */
	label = default_label_create_on_table(STR_USE_HEURISTICS, option_table, 0, 2, 6,7);
	heur_1_button = radio_create_on_table(NULL, STR_YES,GTK_SIGNAL_FUNC(flag_heur_callback), "0",
										INACTIVE, option_table, 2, 3, 6, 7);
	heuristics_group = gtk_radio_button_get_group(GTK_RADIO_BUTTON (heur_1_button));
	heur_2_button = radio_create_on_table(heur_1_button, STR_NO,GTK_SIGNAL_FUNC(flag_heur_callback), "1",
										INACTIVE, option_table, 3, 4, 6, 7);
	heur_3_button = radio_create_on_table(heur_2_button, STR_ONLY,GTK_SIGNAL_FUNC(flag_heur_callback), "2",
										INACTIVE, 	option_table, 4, 5, 6, 7);
#ifdef USE_NEW_TOOLTIP	
	gtk_widget_set_tooltip_text(heur_3_button, STR_NOHEUR_TOOLTIP);
#else
	onlyheur_tooltip=gtk_tooltips_new();
	gtk_tooltips_set_tip (GTK_TOOLTIPS (onlyheur_tooltip),heur_3_button,STR_NOHEUR_TOOLTIP,NULL);
#endif
	button3 = button_create_on_table("F6 "STR_VIRUS_LIST,GTK_SIGNAL_FUNC(fprot_virlist)	, NULL, table1, 7, 8,  20, 21);
	button4 = button_create_on_table("F5 "STR_INFO		,GTK_SIGNAL_FUNC(fprot_info)	, NULL, table1, 6, 7,  20, 21);
	button5 = button_create_on_table("F4 "STR_TEST		,GTK_SIGNAL_FUNC(fprot_test)	, NULL, table1, 5, 6,  20, 21);
	button6 = button_create_on_table("F3 "STR_REPORT	,GTK_SIGNAL_FUNC(view_report)	, NULL, table1, 4, 5,  20, 21);
	button7 = button_create_on_table("F2 "STR_UPDATE	,GTK_SIGNAL_FUNC(fprot_update)	, NULL, table1, 3, 4,  20, 21);
	button8 = button_create_on_table("F1 "STR_SCAN		,GTK_SIGNAL_FUNC(scan_f)		, NULL, table1, 2, 3,  20, 21);

	/* DON'T REMOVE THIS*/
	gtk_widget_show_all(main_window);
	/* GUI Setup */
	if (saved_settings) {
		switch (flag_scanmode) {
			/* flag_scanmode_callback sets the value of flag_scanmode */	
			case 0:  /* Report-only */
				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (scan_1_button), TRUE);
				/* This flags depend on scanmode, so set them properly */
				flag_disinf = 0;
				flag_rename = 0;
				flag_delete = 0;
				break;
			case 1: /* Interactive: needs at least one action flag */
				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (scan_2_button), TRUE);
				/* This flags depend on scanmode, so set them properly */
				if ((flag_disinf + flag_rename + flag_delete) == 0) {
					flag_disinf = 1;
				}
				break;
			case 2: /* Automatic: needs at least one action flag  */
				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (scan_3_button), TRUE);
				/* This flags depend on scanmode, so set them properly */
				if ((flag_disinf + flag_rename + flag_delete) == 0) {
					flag_disinf = 1;
				}
				break;
		}
		switch (flag_macro) {
			/* flag_macro_callback sets the value of flag_macro */	
			case 0:
				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio_button1), TRUE);
				break;
			case 1:
				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio_button2), TRUE);
				break;
			case 2:
				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio_button3), TRUE);
				break;
		}
		if (flag_macro != 1 /* No */) {
			switch (flag_macro_action) {
				/* flag_macro_callback sets the value of flag_macro */	
				case 0:
					gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (macro_1_button), TRUE);
					break;
				case 1:
					gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (macro_2_button), TRUE);
					break;
				case 2:
					gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (macro_3_button), TRUE);
					break;
			}
		}
		switch (flag_file_types) {
			/* flag_file_types_callback sets the value of flag_file_types */	
			case 1:
				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio_button5), TRUE);
				break;
			case 2:
				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio_button6), TRUE);
				break;
		}
		/* Check boxes and radio buttons with simple dependecies */
		switch (flag_heur) {
			/* flag_heur_callback sets the value of flag_heur */
			case 0:
				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (heur_1_button), TRUE);
				if (flag_ai) {
					flag_ai = 0;
					gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (ai_check_button), TRUE);
				}
				break;
			case 1:
				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (heur_2_button), TRUE);
				break;
			case 2:
				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (heur_3_button), TRUE);
				break;
		}
		if (flag_archives) {
			flag_archives = 0;
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (archives_button), TRUE);
			if (have_version >= 440 && flag_passwd_archives) {
				flag_passwd_archives = 0;
				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (passwd_archives_button), TRUE);
			}
		}
		/* This flag depends on flag_scanmode */
		if (flag_disinf) {
			flag_disinf = 0;
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (disinfect_button), TRUE);
		}
		/* This flag depends on flag_scanmode */
		if (flag_rename) {
			flag_rename = 0;
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rename_button), TRUE);
		}
		/* This flag depends on flag_scanmode */
		if (flag_delete) {
			flag_delete = 0;
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (delete_button), TRUE);
		}
		if (flag_scanmode == 0) {
			gtk_widget_hide(action_table);
			gtk_widget_hide(macro_remove_table);
		} else if (flag_macro == 1) {
			gtk_widget_hide(macro_remove_table);	
		}

		/* Simple check boxes */
		if (flag_nosub) {
			flag_nosub = 0;
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check_button1), TRUE);
		}
		if (flag_append) {
			flag_append = 0;
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check_button2), TRUE);
		}
		if (flag_list) {
			flag_list = 0;
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check_button3), TRUE);
		}
		if (have_version < 440 && flag_comp_exe) {
			flag_comp_exe = 0;
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check_button4), TRUE);
		}
		if (flag_collect) {
			flag_collect = 0;
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check_button5), TRUE);
		}
		if (flag_follow) {
			flag_follow = 0;
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check_button6), TRUE);
		}
		if (flag_old) {
			flag_old = 0;
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check_button7), TRUE);
		}
	} else {
		toggle1_f();
		toggle2_f();
	}

	debug_err_msg("");
	debug_err_msg("Settings at startup");
	debug_err_msg("scanmode=%d",     flag_scanmode);
	debug_err_msg("macro=%d",        flag_macro);
	debug_err_msg("macro_action=%d", flag_macro_action);
	debug_err_msg("file_types=%d",   flag_file_types);
	debug_err_msg("heur=%d",         flag_heur);
	debug_err_msg("nosub=%d",        flag_nosub);
	debug_err_msg("disinf=%d",       flag_disinf);
	debug_err_msg("rename=%d",       flag_rename); 
	debug_err_msg("delete=%d",       flag_delete);
	debug_err_msg("append=%d",       flag_append);
	debug_err_msg("list=%d",         flag_list);
	debug_err_msg("archives=%d",     flag_archives);
	if (have_version >= 440) {
		debug_err_msg("arch_passwd=%d", flag_passwd_archives);
	} else {
		/* comp_exe only for f-prot version prior to 4.4.1 */
		debug_err_msg("comp_exe=%d", flag_comp_exe); 
	}
	debug_err_msg("collect=%d",      flag_collect);
	debug_err_msg("ai=%d",           flag_ai);
	debug_err_msg("follow=%d",       flag_follow);
	debug_err_msg("old=%d",          flag_old);

	/* display license window, this can be disabled */
	if (lstat(NO_SPLASH_FILE,&stat_p) != 0) {
		fd = open_file(XFPROT_BIN_DIR LICENSE_FILE);
		if (about_window_create(VERSION, 620, 430,8,
								VERSION"\n"COPYRIGHT"\n"STR_LICENSE"\n"HOMEPAGE"\n"STR_WARRANTY,
								fd,FIRST_TIME) == 1) {
			create_file(NO_SPLASH_FILE);
		}
		xfclose_nostdin(fd);
	}

	gtk_main();
	unixsignal_cleanup();
	exit(EXIT_SUCCESS);
}
